@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2001 Mikael Hallendal <micke@codefactory.se>
+ * Copyright (C) 2001-2002 Mikael Hallendal <micke@codefactory.se>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -28,21 +28,28 @@
 #include "yelp-section.h"
 
 YelpSection *
-yelp_section_new (YelpSectionType type,
-		  const gchar    *name,
-		  const gchar    *uri,
-		  const gchar    *reference,
-		  const gchar    *scheme)
+yelp_section_new (YelpSectionType  type,
+		  const gchar     *name,
+		  YelpURI         *uri)
 {
 	YelpSection *section;
 
 	section = g_new0 (YelpSection, 1);
 
 	section->type      = type;
-	section->name      = g_strdup (name);
-	section->uri       = g_strdup (uri);
-	section->reference = g_strdup (reference);
-	section->scheme    = g_strdup (scheme);
+
+	if (name) {
+		section->name = g_strdup (name);
+	} else {
+		section->name = g_strdup ("");
+	}
+	
+	if (uri) {
+		section->uri = yelp_uri_ref (uri);
+	} else {
+		section->uri = NULL;
+	}
+	
 	
 	return section;
 }
@@ -52,9 +59,7 @@ yelp_section_copy (const YelpSection *section)
 {
 	return yelp_section_new (section->type, 
 				 section->name, 
-				 section->uri, 
-				 section->reference,
-				 section->scheme);
+				 section->uri);
 }
 
 void 
@@ -65,11 +70,7 @@ yelp_section_free (YelpSection *section)
 	}
 	
 	if (section->uri) {
-		g_free (section->uri);
-	}
-	
-	if (section->reference) {
-		g_free (section->reference);
+		yelp_uri_unref (section->uri);
 	}
 }
 
